@@ -180,6 +180,7 @@ static BROWSE_LIST_ENTRY* browse_job_from_queue ()
 static void browse_job_start_from_entry (BROWSE_LIST_ENTRY *entry)
 {
   BROWSE_INFO *browse_info;
+  int port = -1;
   
   if ( entry->r_entry.type & BROWSE_USER_MASK )
   {
@@ -192,6 +193,11 @@ static void browse_job_start_from_entry (BROWSE_LIST_ENTRY *entry)
     passwd = dummy;
   }
   
+  if ( pref.v.use_smb_port )
+  {
+    port = pref.v.smb_port;
+  }
+  
   browse_info = (BROWSE_INFO*)g_malloc(sizeof(BROWSE_INFO));
   browse_info->g_info.mode = entry->r_entry.mode;
   browse_info->g_info.browse_id = browse_jobs;
@@ -213,6 +219,7 @@ static void browse_job_start_from_entry (BROWSE_LIST_ENTRY *entry)
                            passwd,
                            "",
                            1,
+                           port,
                            &(browse_info->g_info));
           }
           else
@@ -230,6 +237,7 @@ static void browse_job_start_from_entry (BROWSE_LIST_ENTRY *entry)
                            passwd,
                            entry->g_entry.browser,
                            0,
+                           port,
                            &(browse_info->g_info));
           }
           else
@@ -247,6 +255,7 @@ static void browse_job_start_from_entry (BROWSE_LIST_ENTRY *entry)
                            passwd,
                            "",
                            0,
+                           port,
                            &(browse_info->g_info));
           }
           else
@@ -268,6 +277,7 @@ static void browse_job_start_from_entry (BROWSE_LIST_ENTRY *entry)
                           usr,
                           passwd,
                           entry->h_entry.domain,
+                          port,
                           &(browse_info->h_info));
         }
         else
@@ -285,6 +295,7 @@ static void browse_job_start_from_entry (BROWSE_LIST_ENTRY *entry)
                          entry->s_entry.ip,
                          usr,
                          passwd,
+                         port,
                          &(browse_info->s_info));
         }
         else
@@ -514,7 +525,12 @@ static void browse_groups_start ( char *master_browser,
   GROUPS_BROWSE_INFO *browse_info;
   BROWSE_LIST_ENTRY *entry;
   char *group_name, *usr, *passwd, *wins_name;
+  int port = -1;
   
+  if ( pref.v.use_smb_port )
+  {
+    port = pref.v.smb_port;
+  }
   group_name = pref.v.workgroup;
   usr = cempty;
   passwd = cempty;
@@ -544,6 +560,7 @@ static void browse_groups_start ( char *master_browser,
                      passwd,
                      wins_name,
                      first_master,
+                     port,
                      browse_info);
     }
     else
@@ -650,6 +667,10 @@ static void browse_machines_done ( char *group_name,
       }
     }
   }
+  if ( !(mode & BROWSE_MODE_NETWORK) )
+  {
+    gui_set_cursor(cursor_normal);
+  }
 }
 
 static void browse_machines_start ( char *group_name,
@@ -660,6 +681,7 @@ static void browse_machines_start ( char *group_name,
   MACHINES_BROWSE_INFO *browse_info;
   BROWSE_LIST_ENTRY *entry;
   char *usr, *passwd, *domain;
+  int port = -1;
   
   usr = cempty;
   passwd = cempty;
@@ -668,6 +690,11 @@ static void browse_machines_start ( char *group_name,
   if ( pref.v.use_group_browse )
     domain = group_name;
   
+  if ( pref.v.use_smb_port )
+  {
+    port = pref.v.smb_port;
+  }
+  
   if ( mode & BROWSE_MODE_USER )
   {
     usr = globals.browse_user;
@@ -691,6 +718,7 @@ static void browse_machines_start ( char *group_name,
                        usr,
                        passwd,
                        domain,
+                       port,
                        browse_info);
     }
     else
@@ -800,11 +828,17 @@ static void browse_shares_start ( char *group_name,
   SHARES_BROWSE_INFO *browse_info;
   BROWSE_LIST_ENTRY *entry;
   char *usr, *passwd, *domain;
+  int port = -1;
   
   usr = cempty;
   passwd = cempty;
   domain = cempty;
   
+  if ( pref.v.use_smb_port )
+  {
+    port = pref.v.smb_port;
+  }
+  
   if ( pref.v.use_group_browse )
     domain = domain_name;
   
@@ -830,6 +864,7 @@ static void browse_shares_start ( char *group_name,
                      ip,
                      usr,
                      passwd,
+                     port,
                      browse_info);
     }
     else
