@@ -62,8 +62,10 @@
 
 /* path definitions */
 
-#define GLOBAL_PATH             "/linneighborhood/"            /* global configuration path */
-#define LOCAL_PATH              ".linneighborhood/"            /* local configuration path, relative from $HOME */
+#define GLOBAL_PATH             "/LinNeighborhood/"            /* global configuration path */
+#define GLOBAL_PATH_DPCTD       "/linneighborhood/"            /* old (deprecated) global configuration path */
+#define LOCAL_PATH              ".LinNeighborhood/"            /* local configuration path, relative from $HOME */
+#define LOCAL_PATH_DPCTD        ".linneighborhood/"            /* old (deprecated) local configuration path */
 
 #define PREF_FILE               "preferences"                  /* name of preferences file */
 #define SIZE_FILE               "size"                         /* name of the file for the window geometry */
@@ -123,6 +125,7 @@ typedef unsigned int BROWSE_MODE;
 
 #define MASTER_LEN                63    /* often used lengths */
 #define PATH_LEN                 255
+#define ARG_LEN                  255
 #define MAX_REC_LEN              127
 #define SERVICE_LEN               63    /* max length of service */
 #define USER_LEN                  25
@@ -475,6 +478,8 @@ typedef struct _pref_value_struct {
         unsigned char delete_mountpoints;           /* delete the mountpoint on unmount */
         unsigned char replace_space_mount;          /* replace spaces in mount path with underscores */
         unsigned char mount_default_no_dialog;      /* don't show the mount dialog, use defaults */
+        unsigned char use_smb_port;                 /* use the dedicated smb port */
+        int smb_port;                               /* dedicated smb port */
         SMBMOUNT smbmount_version;                  /* smbmount from samba x.x.x */
         char root_mnt_dir[PATH_LEN+1];              /* default root mount directory */
         char filemanager_exe[PATH_LEN+1];           /* file manager command */
@@ -482,6 +487,10 @@ typedef struct _pref_value_struct {
         char nmblookup_exe[PATH_LEN+1];             /* command path nmblookup */
         char smbmount_exe[PATH_LEN+1];              /* command path smbmount */
         char smbumount_exe[PATH_LEN+1];             /* command path smbumount */
+        char smbclient_arg[ARG_LEN+1];              /* additional arguments for smbclient */
+        char nmblookup_arg[ARG_LEN+1];              /*  - " -  nmblookup */
+        char smbmount_arg[ARG_LEN+1];               /*  - " -  smbmount */
+        char smbumount_arg[ARG_LEN+1];              /*  - " -  smbumount */
         char default_user[USER_LEN+1];              /* default user for browsing */
 } pref_value_struct;
 
@@ -507,6 +516,8 @@ typedef struct _pref_bool_struct {
         unsigned char delete_mountpoints;
         unsigned char replace_space_mount;
         unsigned char mount_default_no_dialog;
+        unsigned char use_smb_port;
+        unsigned char smb_port;
         unsigned char smbmount_version;
         unsigned char root_mnt_dir;
         unsigned char filemanager_exe;
@@ -514,6 +525,10 @@ typedef struct _pref_bool_struct {
         unsigned char nmblookup_exe;
         unsigned char smbmount_exe;
         unsigned char smbumount_exe;
+        unsigned char smbclient_arg;
+        unsigned char nmblookup_arg;
+        unsigned char smbmount_arg;
+        unsigned char smbumount_arg;
         unsigned char default_user;
 } pref_bool_struct;
 
