@@ -536,6 +536,11 @@ static void remove_fav_machines_callback()
                     remove_fav_machines_msgdlg_callback);
 }
 
+static void preferences_save_callback()
+{
+  preferences_save(&pref);
+}
+
 static void separator (GtkWidget * menu)
 {
   GtkWidget *item;
@@ -617,6 +622,20 @@ static void main_menu_create (GtkWidget * vbox)
   gtk_signal_connect_object( GTK_OBJECT(menu_item), "activate",
                              GTK_SIGNAL_FUNC(remove_fav_machines_callback), NULL);
   gtk_widget_show(menu_item);
+  
+  separator(main_menu);
+  
+  menu_item = gtk_menu_item_new_with_label(_("Preferences..."));
+  gtk_menu_append( GTK_MENU(main_menu), menu_item);
+  gtk_signal_connect_object( GTK_OBJECT(menu_item), "activate",
+                             GTK_SIGNAL_FUNC(preferences_dialog), NULL);
+  gtk_widget_show(menu_item);
+  
+  menu_item = gtk_menu_item_new_with_label(_("Save Preferences"));
+  gtk_menu_append( GTK_MENU(main_menu), menu_item);
+  gtk_signal_connect_object( GTK_OBJECT(menu_item), "activate",
+                             GTK_SIGNAL_FUNC(preferences_save_callback), NULL);
+  gtk_widget_show(menu_item);
 
   gtk_menu_item_set_submenu( GTK_MENU_ITEM(main_item), main_menu);
   gtk_menu_bar_append( GTK_MENU_BAR(menu_bar), main_item);
@@ -654,17 +673,8 @@ static void main_menu_create (GtkWidget * vbox)
   gtk_signal_connect_object( GTK_OBJECT(menu_item), "activate",
                              GTK_SIGNAL_FUNC(export_network_image_dialog), NULL);
   gtk_widget_show(menu_item);
-  
-  separator(main_menu);
-  
-  menu_item = gtk_menu_item_new_with_label(_("Preferences..."));
-  gtk_menu_append( GTK_MENU(main_menu), menu_item);
-  gtk_signal_connect_object( GTK_OBJECT(menu_item), "activate",
-                             GTK_SIGNAL_FUNC(preferences_dialog), NULL);
-  gtk_widget_show(menu_item);
 
   gtk_menu_item_set_submenu( GTK_MENU_ITEM(main_item), main_menu);
-
   gtk_menu_bar_append( GTK_MENU_BAR(menu_bar), main_item);
 
   /* -- Help menu -- */
@@ -680,7 +690,6 @@ static void main_menu_create (GtkWidget * vbox)
   gtk_widget_show(menu_item);
 
   gtk_menu_item_set_submenu( GTK_MENU_ITEM(main_item), main_menu);
-
   gtk_menu_bar_append( GTK_MENU_BAR(menu_bar), main_item);
 }
 
