@@ -330,7 +330,7 @@ static void file_enumerate_prefs (char *file, file_enumerate_prefs_callback call
 /* set the preferences struct to default values */
 static void preferences_to_default (pref_struct *pref)
 {
-  strcpy(pref->v.workgroup,"workgroup");
+  strcpy(pref->v.workgroup,"-");
   strcpy(pref->v.master,cempty);
   strcpy(pref->v.wins_server,cempty);
   pref->v.master_browser_scan = NO;
@@ -351,6 +351,8 @@ static void preferences_to_default (pref_struct *pref)
   pref->v.delete_mountpoints = NO;
   pref->v.replace_space_mount = NO;
   pref->v.mount_default_no_dialog = NO;
+  pref->v.use_smb_port = NO;
+  pref->v.smb_port = 139;
   pref->v.smbmount_version = SMBMOUNT_206;
   string_ncopy(pref->v.root_mnt_dir, globals.home_dir, PATH_LEN); /* default root mount directory */
   string_ncat(pref->v.root_mnt_dir, "mnt/", PATH_LEN);
@@ -359,12 +361,16 @@ static void preferences_to_default (pref_struct *pref)
   strcpy(pref->v.nmblookup_exe, "nmblookup");
   strcpy(pref->v.smbmount_exe, "smbmount");
   strcpy(pref->v.smbumount_exe, "smbumount");
+  pref->v.smbclient_arg[0] = 0;
+  pref->v.nmblookup_arg[0] = 0;
+  pref->v.smbmount_arg[0] = 0;
+  pref->v.smbumount_arg[0] = 0;
   pref->v.default_user[0] = 0;
 
   memset(&(pref->b), 0, sizeof(pref_bool_struct));
 }
 
-#define TOKEN_PREF_NUM      29
+#define TOKEN_PREF_NUM      35
 
 static char *token_pref_names[TOKEN_PREF_NUM] = {
         "quick_browse",                    /* 0 */
@@ -395,7 +401,13 @@ static char *token_pref_names[TOKEN_PREF_NUM] = {
         "default_user",                    /* 25 */
         "delete_mountpoints",              /* 26 */
         "replace_space_mount",             /* 27 */
-        "mount_default_no_dialog"          /* 28 */
+        "mount_default_no_dialog",         /* 28 */
+        "use_smb_port",                    /* 29 */
+        "smb_port",                        /* 30 */
+        "smbclient_arg",                   /* 31 */
+        "nmblookup_arg",                   /* 32 */
+        "smbmount_arg",                    /* 33 */
+        "smbumount_arg",                   /* 34 */
 };
 
 static unsigned char preferences_enumerate_callback (char *token, char *value, gpointer data)
@@ -572,6 +584,43 @@ static unsigned char preferences_enumerate_callback (char *token, char *value, g
             pref->v.mount_default_no_dialog = !strcmp(value, yes_statement);
             pref->b.mount_default_no_dialog = YES;
             break;
+        /* use_smb_port */
+        case 29:
+            pref->v.use_smb_port = !strcmp(value, yes_statement);
+            pref->b.use_smb_port = YES;
+            break;
+        /* smb_port */
+        case 30:
+            ivalue = (int)(strtol(value, &ptemp, 10));
+            if ( *ptemp == 0 )
+            {
+              if ( (ivalue > 0) && (ivalue < 65536) )
+              {
+                pref->v.smb_port = ivalue;
+                pref->b.smb_port = YES;
+              }
+            }
+            break;
+        /* smbclient_arg */
+        case 31:
+            string_ncopy(pref->v.smbclient_arg, value, ARG_LEN);
+            pref->b.smbclient_arg = YES;
+            break;
+        /* nmblookup_arg */
+        case 32:
+            string_ncopy(pref->v.nmblookup_arg, value, ARG_LEN);
+            pref->b.nmblookup_arg = YES;
+            break;
+        /* smbmount_arg */
+        case 33:
+            string_ncopy(pref->v.smbmount_arg, value, ARG_LEN);
+            pref->b.smbmount_arg = YES;
+            break;
+        /* smbumount_arg */
+        case 34:
+            string_ncopy(pref->v.smbumount_arg, value, ARG_LEN);
+            pref->b.smbumount_arg = YES;
+            break;
       }
       break;
     }
@@ -580,16 +629,18 @@ static unsigned char preferences_enumerate_callback (char *token, char *value, g
   return 0;
 }
 
-static unsigned char preferences_smb_conf_callback (char *token, char *value, gpointer data)
+/* not needed, workgroup preset to "-" */
+
+/*static unsigned char preferences_smb_conf_callback (char *token, char *value, gpointer data)
 {
   pref_struct *pref;
 
   pref = (pref_struct*)data;
   if ( pref == NULL )
-    return 0;
+    return 0;*/
   
   /* workgroup token from smb.conf ? */
-  if ( !strcmp(token, "workgroup") )
+/*  if ( !strcmp(token, "workgroup") )
   {
     string_ncopy(pref->v.workgroup, value, MAXGROUPNAMEL);
     pref->b.workgroup = YES;
@@ -597,17 +648,19 @@ static unsigned char preferences_smb_conf_callback (char *token, char *value, gp
   }
   
   return 0;
-}
+}*/
 
 static void preferences_load_from_file (char *file, pref_struct *pref)
 {
   file_enumerate_prefs(file, preferences_enumerate_callback, pref);
 }
 
-static void preferences_load_smb_conf (char *file, pref_struct *pref)
+/* not needed, workgroup preset to "-" */
+
+/*static void preferences_load_smb_conf (char *file, pref_struct *pref)
 {
   file_enumerate_prefs(file, preferences_smb_conf_callback, pref);
-}
+}*/
 
 void preferences_save (pref_struct *pref)
 {
@@ -731,6 +784,24 @@ void preferences_save (pref_struct *pref)
   /* mount_default_no_dialog */
   token_write_boolean(pref->b.mount_default_no_dialog, pref->v.mount_default_no_dialog, token_pref_names[28], fd);
   
+  /* use_smb_port */
+  token_write_boolean(pref->b.use_smb_port, pref->v.use_smb_port, token_pref_names[29], fd);
+  
+  /* smb_port */
+  token_write_integer(pref->b.smb_port, pref->v.smb_port, token_pref_names[30], fd);
+  
+  /* smbclient_arg */
+  token_write_string(pref->b.smbclient_arg, pref->v.smbclient_arg, token_pref_names[31], fd);
+  
+  /* nmblookup_arg */
+  token_write_string(pref->b.nmblookup_arg, pref->v.nmblookup_arg, token_pref_names[32], fd);
+  
+  /* smbmount_arg */
+  token_write_string(pref->b.smbmount_arg, pref->v.smbmount_arg, token_pref_names[33], fd);
+  
+  /* smbumount_arg */
+  token_write_string(pref->b.smbumount_arg, pref->v.smbumount_arg, token_pref_names[34], fd);
+  
   fclose(fd);
 }
 
@@ -743,7 +814,12 @@ void preferences_load (pref_struct *pref)
   
   preferences_to_default(pref);
   /* try to load global preferences */
-  string_ncopy(path, LIBDIR GLOBAL_PATH, PATH_LEN);
+  string_ncopy(path, DATADIR GLOBAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, LIBDIR GLOBAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PREF_FILE, PATH_LEN);
   preferences_load_from_file(path, pref);
   if ( pref->b.workgroup )
@@ -751,6 +827,12 @@ void preferences_load (pref_struct *pref)
   /* try to load local preferences */
   string_ncopy(path, globals.home_dir, PATH_LEN);
   string_ncat(path, LOCAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, globals.home_dir, PATH_LEN);
+    string_ncat(path, LOCAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PREF_FILE, PATH_LEN);
   /* clear the flags to recognize local preferences */
   memset(&(pref->b), 0, sizeof(pref_bool_struct));
@@ -758,8 +840,14 @@ void preferences_load (pref_struct *pref)
   if ( pref->b.workgroup )
     workgroup_loaded = 1;
   
-  /* if workgroup not set, try to load smb.conf */
+  /* new strategy: if workgroup not set, set it to "-" */
   if ( !workgroup_loaded )
+  {
+    strcpy(pref->v.workgroup, "-");
+  }
+  
+  /* if workgroup not set, try to load smb.conf */
+  /*if ( !workgroup_loaded )
   {
     strcpy(path, "/etc/smb.conf");
     preferences_load_smb_conf(path, pref);
@@ -768,7 +856,7 @@ void preferences_load (pref_struct *pref)
       strcpy(path, "/usr/local/samba/lib/smb.conf");
       preferences_load_smb_conf(path, pref);
     }
-  }
+  }*/
 }
 
 /* ------------------------------------------------------------------------ */
@@ -879,6 +967,12 @@ void size_load (size_struct *size)
   /* try to load size information */
   string_ncopy(path, globals.home_dir, PATH_LEN);
   string_ncat(path, LOCAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, globals.home_dir, PATH_LEN);
+    string_ncat(path, LOCAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, SIZE_FILE, PATH_LEN);
   size_load_from_file(path, size);
 }
@@ -966,7 +1060,12 @@ void prefhost_load (void)
   notify_stop();
   
   /* try to load global preferred hosts */
-  string_ncopy(path, LIBDIR GLOBAL_PATH, PATH_LEN);
+  string_ncopy(path, DATADIR GLOBAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, LIBDIR GLOBAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PREF_FILE, PATH_LEN);
 
   hosttype = host_global;
@@ -975,6 +1074,12 @@ void prefhost_load (void)
   /* try to load local preferred hosts */
   string_ncopy(path, globals.home_dir, PATH_LEN);
   string_ncat(path, LOCAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, globals.home_dir, PATH_LEN);
+    string_ncat(path, LOCAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, HOST_FILE, PATH_LEN);
 
   hosttype = host_local;
@@ -1070,7 +1175,12 @@ void add_master_load ()
   notify_stop();
   
   /* try to load global master browsers */
-  string_ncopy(path, LIBDIR GLOBAL_PATH, PATH_LEN);
+  string_ncopy(path, DATADIR GLOBAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, LIBDIR GLOBAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PREF_FILE, PATH_LEN);
 
   masterbrowsertype = master_browser_global;
@@ -1079,6 +1189,12 @@ void add_master_load ()
   /* try to load local master browsers */
   string_ncopy(path, globals.home_dir, PATH_LEN);
   string_ncat(path, LOCAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, globals.home_dir, PATH_LEN);
+    string_ncat(path, LOCAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, MASTER_FILE, PATH_LEN);
 
   masterbrowsertype = master_browser_local;
@@ -1181,7 +1297,12 @@ void groupmaster_load ()
   notify_stop();
   
   /* try to load global master browsers */
-  string_ncopy(path, LIBDIR GLOBAL_PATH, PATH_LEN);
+  string_ncopy(path, DATADIR GLOBAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, LIBDIR GLOBAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PREF_FILE, PATH_LEN);
 
   groupmastertype = type_user_add_global;
@@ -1190,6 +1311,12 @@ void groupmaster_load ()
   /* try to load local master browsers */
   string_ncopy(path, globals.home_dir, PATH_LEN);
   string_ncat(path, LOCAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, globals.home_dir, PATH_LEN);
+    string_ncat(path, LOCAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, GROUPMASTER_FILE, PATH_LEN);
 
   groupmastertype = type_user_add_local;
@@ -1304,7 +1431,12 @@ GSList *mountpoint_load ()
   mplist = (GSList*)NULL;
   
   /* try to load global mountpoints */
-  /*string_ncopy(path, LIBDIR GLOBAL_PATH, PATH_LEN);
+  /*string_ncopy(path, DATADIR GLOBAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {*/
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    /*string_ncopy(path, LIBDIR GLOBAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PREF_FILE, PATH_LEN);
 
   mountpoint_load_from_file(path, &mplist);*/
@@ -1312,6 +1444,12 @@ GSList *mountpoint_load ()
   /* try to load local mountpoints */
   string_ncopy(path, globals.home_dir, PATH_LEN);
   string_ncat(path, LOCAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, globals.home_dir, PATH_LEN);
+    string_ncat(path, LOCAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, MOUNTPOINT_FILE, PATH_LEN);
 
   mountpoint_load_from_file(path, &mplist);
@@ -1406,7 +1544,12 @@ void password_load ()
   char path[PATH_LEN+1];
 
   /* try to load global password */
-  string_ncopy(path, LIBDIR GLOBAL_PATH, PATH_LEN);
+  string_ncopy(path, DATADIR GLOBAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, LIBDIR GLOBAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PASSWORD_FILE, PATH_LEN);
 
   password_load_from_file(path);
@@ -1414,6 +1557,12 @@ void password_load ()
   /* try to load local mountpoints */
   string_ncopy(path, globals.home_dir, PATH_LEN);
   string_ncat(path, LOCAL_PATH, PATH_LEN);
+  if ( !path_exist(path) )
+  {
+    /* path has changed since 0.6.5, try to load the old one to be compatible */
+    string_ncopy(path, globals.home_dir, PATH_LEN);
+    string_ncat(path, LOCAL_PATH_DPCTD, PATH_LEN);
+  }
   string_ncat(path, PASSWORD_FILE, PATH_LEN);
 
   password_load_from_file(path);
