@@ -48,7 +48,8 @@ typedef struct _GBROWSE_STRUCT{
   char linebuf[MAXSMBLINEL+1];
   char masterbrowser[MAXSMBLINEL+1];
   char userpasswdstr[MAXLOGSTRLEN+1],userdummystr[MAXLOGSTRLEN+1];
-  GROUPMASTER_STRUCT group_master;  
+  GROUPMASTER_STRUCT group_master;
+  int port;
   int  linepos,linenum,firstbrowse,foundsecondentries;
   int  group1pos,master1pos; 
   GSList *groupmasterlist;
@@ -63,8 +64,8 @@ typedef struct _HBROWSE_STRUCT{
   pid_t childpid;
   gint input_tag;
   char linebuf[MAXSMBLINEL+1];
-  GROUPMASTER_STRUCT group_master;  
-/*  char group_name[MAXGROUPNAMEL+1]; */
+  GROUPMASTER_STRUCT group_master;
+  int port;
   int  linepos,linenum;
   int  machine1pos,comment1pos; 
   GSList *machinecommentlist;
@@ -80,6 +81,7 @@ typedef struct _SBROWSE_STRUCT{
   char linebuf[MAXSMBLINEL+1];
   char group_name[MAXGROUPNAMEL+1],machine_name[MAXMACHNAMEL+1];
   char user[USER_LEN+1], password[PASSWORD_LEN+1];
+  int port;
   int  linepos,linenum;
   int  share1pos,shtype1pos,comment1pos;
   GSList *sharecommenttypelist;
@@ -109,8 +111,6 @@ static GSList *gbrowsedatlist=NULL,
               *lookupbyiplist=NULL;
 
 
-/*char smbclientExe[PATH_MAX] = "smbclient",
-     nmblookupExe[PATH_MAX] = "nmblookup";*/
 char nomaster[] = "?nomaster?";
 
 unsigned  char sorted_sharelist = TRUE;
@@ -305,6 +305,22 @@ static void GetSMBGroups_inputcallback(GBROWSE_STRUCT *browsedat,gint source, Gd
   GetSMBStateType smbstate;
   GSList *gmlist;
   char logstr[MAXLOGSTRLEN+1];
+  char port_str[16];
+  char *cport = NULL;
+  char *cportkey = NULL;
+  char *cArg[20];
+  char *cAddArg[10];
+  int argcount = 0;
+  char add_arg_string[ARG_LEN+1];
+  int add_argcount = 0;
+  int loop;
+  
+  if ( (browsedat->port >= 0) && (browsedat->port < 65536) )
+  {
+    cportkey = "-p";
+    sprintf(port_str, "%d", browsedat->port);
+    cport = port_str;
+  }
   
   if(cond & GDK_INPUT_READ_)
   {
@@ -426,6 +442,11 @@ static void GetSMBGroups_inputcallback(GBROWSE_STRUCT *browsedat,gint source, Gd
       }
       if(browsedat->firstbrowse)  /*Configured masterbrowser is not masterbrowser */
       {                           /*browse again with found masterbrowser */
+       if ( !is_empty_string(pref.v.smbclient_arg) )
+       {
+         string_ncopy(add_arg_string, pref.v.smbclient_arg, ARG_LEN);
+         add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+       }
        browsedat->linepos=0;
        browsedat->linenum=0;
        browsedat->browse_state=search_head;
@@ -444,19 +465,53 @@ static void GetSMBGroups_inputcallback(GBROWSE_STRUCT *browsedat,gint source, Gd
           if (browsedat->childpid ==0)  
           {
            close(browsedat->fd[0]);   
-           dup2(browsedat->fd[1],STDOUT_FILENO); 
-           execlp(pref_get_smbclient_exe(),
+           dup2(browsedat->fd[1],STDOUT_FILENO);
+           
+           cArg[argcount++] = "smbclient";
+           cArg[argcount++] = "-L";
+           cArg[argcount++] = ((GROUPMASTER_STRUCT*)(gmlist->data))->master_browser;
+           cArg[argcount++] = "-W";
+           cArg[argcount++] = ((GROUPMASTER_STRUCT*)(gmlist->data))->group_name;
+           cArg[argcount++] = browsedat->userpasswdstr;
+           cArg[argcount++] = "-d1";
+           if ( (cportkey != NULL) && (cport && NULL) ) {
+            cArg[argcount++] = cportkey;
+            cArg[argcount++] = cport;
+           }
+           for ( loop = 0; loop < add_argcount; loop++ )
+           {
+             cArg[argcount++] = cAddArg[loop];
+           }
+           cArg[argcount++] = NULL;
+           
+           execvp(pref_get_smbclient_exe(), cArg);
+         
+           /*execlp(pref_get_smbclient_exe(),
                   "smbclient",
                   "-L",((GROUPMASTER_STRUCT*)(gmlist->data))->master_browser,
                   "-W",((GROUPMASTER_STRUCT*)(gmlist->data))->group_name,
                   browsedat->userpasswdstr,
                   "-d1",
-                  NULL);
+                  cportkey,
+                  cport,
+                  NULL);*/
            _exit(0);
           }
           sprintf(logstr,"GetSMBGroups2:%s -L %s -W %s %s -d1",pref_get_smbclient_exe(),
                     ((GROUPMASTER_STRUCT*)(gmlist->data))->master_browser,
                     ((GROUPMASTER_STRUCT*)(gmlist->data))->group_name,browsedat->userdummystr);
+          if ( (cport != NULL) && (cportkey != NULL) )
+          {
+            string_ncat(logstr, " ", MAXLOGSTRLEN);
+            string_ncat(logstr, cportkey, MAXLOGSTRLEN);
+            string_ncat(logstr, " ", MAXLOGSTRLEN);
+            string_ncat(logstr, cport, MAXLOGSTRLEN);
+          }
+          for ( loop = 0; loop < add_argcount; loop++ )
+          {
+            string_ncat(logstr, " ", MAXLOGSTRLEN);
+            string_ncat(logstr, cAddArg[loop], MAXLOGSTRLEN);
+          }
           gui_log_window_insert_string(logstr,1);
           close(browsedat->fd[1]);    
           browsedat->lasttab=0;
@@ -491,6 +546,23 @@ static void GetSMBGroups_nmbinputcallback(GBROWSE_STRUCT *browsedat,gint source,
   char master_browser[MAXMACHNAMEL+1];
   GetSMBStateType smbstate;
   int nodetype,isgroup,isactive;
+  char port_str[16];
+  char *cport = NULL;
+  char *cportkey = NULL;
+  char *cArg[20];
+  char *cAddArg[10];
+  int argcount = 0;
+  char add_arg_string[ARG_LEN+1];
+  int add_argcount = 0;
+  int loop;
+   
+  if ( (browsedat->port >= 0) && (browsedat->port < 65536) )
+  {
+    cportkey = "-p";
+    sprintf(port_str, "%d", browsedat->port);
+    cport = port_str;
+  }
+  
   if(cond & GDK_INPUT_READ_)
   {
    if (fdread_line_ok2(source,&linec,&(browsedat->linepos),browsedat->linebuf,0,NULL))
@@ -514,7 +586,6 @@ static void GetSMBGroups_nmbinputcallback(GBROWSE_STRUCT *browsedat,gint source,
            linec=EOF;
           }
          }
-/*         else linec=EOF;*/
       default:;   
      }
      browsedat->linepos=0;
@@ -528,6 +599,11 @@ static void GetSMBGroups_nmbinputcallback(GBROWSE_STRUCT *browsedat,gint source,
      gdk_input_remove(browsedat->input_tag); 
      if (found_master)
      {
+       if ( !is_empty_string(pref.v.smbclient_arg) )
+       {
+         string_ncopy(add_arg_string, pref.v.smbclient_arg, ARG_LEN);
+         add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+       }
        if((pipe_nok=pipe(browsedat->fd))|| (browsedat->childpid =fork())== -1)   
          {
            if (!pipe_nok)
@@ -538,20 +614,54 @@ static void GetSMBGroups_nmbinputcallback(GBROWSE_STRUCT *browsedat,gint source,
          } 
        else 
          {
-          if (browsedat->childpid ==0)  
+          if (browsedat->childpid == 0)  
           {
            close(browsedat->fd[0]);   
            dup2(browsedat->fd[1],STDOUT_FILENO); 
-           execlp(pref_get_smbclient_exe(),
+           
+           cArg[argcount++] = "smbclient";
+           cArg[argcount++] = "-L";
+           cArg[argcount++] = master_browser;
+           cArg[argcount++] = "-W";
+           cArg[argcount++] = browsedat->group_master.group_name;
+           cArg[argcount++] = browsedat->userpasswdstr;
+           cArg[argcount++] = "-d1";
+           if ( (cportkey != NULL) && (cport && NULL) ) {
+            cArg[argcount++] = cportkey;
+            cArg[argcount++] = cport;
+           }
+           for ( loop = 0; loop < add_argcount; loop++ )
+           {
+             cArg[argcount++] = cAddArg[loop];
+           }
+           cArg[argcount] = NULL;
+           
+           execvp(pref_get_smbclient_exe(), cArg);
+           
+           /*execlp(pref_get_smbclient_exe(),
                   "smbclient",
                   "-L",master_browser,
                   "-W",browsedat->group_master.group_name,browsedat->userpasswdstr,
                   "-d1",
-                  NULL);
+                  cportkey,
+                  cport,
+                  NULL);*/
            _exit(0);
           }
           sprintf(logstr,"GetSMBGroups:%s -L %s -W %s %s -d1",pref_get_smbclient_exe(),
                     master_browser,browsedat->group_master.group_name,browsedat->userdummystr);
+          if ( (cport != NULL) && (cportkey != NULL) )
+          {
+            string_ncat(logstr, " ", MAXLOGSTRLEN);
+            string_ncat(logstr, cportkey, MAXLOGSTRLEN);
+            string_ncat(logstr, " ", MAXLOGSTRLEN);
+            string_ncat(logstr, cport, MAXLOGSTRLEN);
+          }
+          for ( loop = 0; loop < add_argcount; loop++ )
+          {
+            string_ncat(logstr, " ", MAXLOGSTRLEN);
+            string_ncat(logstr, cAddArg[loop], MAXLOGSTRLEN);
+          }
           gui_log_window_insert_string(logstr,1);
           browsedat->linepos=0;
           browsedat->linenum=0;
@@ -580,44 +690,27 @@ static void GetSMBGroups_nmbinputcallback(GBROWSE_STRUCT *browsedat,gint source,
 }
  
 
-void GetSMBGroups ( char * group_name, char * master_browser, char * user, char * passwd, char * wins_name,char first_master, GROUPS_BROWSE_INFO *browse_info)
+void GetSMBGroups ( char * group_name, char * master_browser, char * user, char * passwd, char * wins_name,char first_master,int port, GROUPS_BROWSE_INFO *browse_info)
 {
-//GSList *gbdlist;
-GBROWSE_STRUCT *browsedat;
-int pipe_nok;
-char logstr[MAXLOGSTRLEN+1];
-char group_type[MAXGROUPNAMEL+1+3];
+  GBROWSE_STRUCT *browsedat;
+  int pipe_nok;
+  char logstr[MAXLOGSTRLEN+1];
+  char group_type[MAXGROUPNAMEL+1+3];
+  char port_str[16];
+  char *cport = NULL;
+  char *cportkey = NULL;
+  char *cArg[20];
+  char *cAddArg[10];
+  int argcount = 0;
+  char add_arg_string[ARG_LEN+1];
+  int add_argcount = 0;
+  int loop;
 
-/*  g_print("GetSMBGroups(%s)\n",master_browser); */
-
-/*  if (gbrowsedatlist)
-  {
-    gbdlist= gbrowsedatlist;
-    while(gbdlist)
-     {
-      if (strcasecmp(((GBROWSE_STRUCT*)(gbdlist->data))->masterbrowser,master_browser))
-      {
-       gbdlist=gbdlist->next;
-      }
-      else  
-      { //if GetSMBGroups with same masterbrowser is running, stop it 
-       browsedat=gbdlist->data;
-       close(browsedat->fd[0]);
-       usleep(50000);
-       kill(browsedat->childpid,SIGINT);
-       waitpid(browsedat->childpid,NULL,0);
-       gdk_input_remove(browsedat->input_tag); 
-       gbrowsedatlist=g_slist_remove(gbrowsedatlist,browsedat);
-       slist_free_with_data(&(browsedat->groupmasterlist));
-       g_free(browsedat);
-       break;
-      }
-     }
-  }   */
   browsedat=g_malloc(sizeof(GBROWSE_STRUCT));
   browsedat->groupmasterlist=NULL;
   browsedat->masterbrowser[MAXSMBLINEL]=0;
   strncpy(browsedat->masterbrowser,master_browser,MAXSMBLINEL);
+  browsedat->port = port;
   browsedat->linepos=0;
   browsedat->linenum=0;
   browsedat->browse_state=search_head;
@@ -632,6 +725,17 @@ char group_type[MAXGROUPNAMEL+1+3];
     strcpy(browsedat->userdummystr,browsedat->userpasswdstr); 
     memset(browsedat->userdummystr+strlen(browsedat->userdummystr)-strlen(passwd),'*',strlen(passwd));
   }
+  if ( (port >= 0) && (port < 65536) )
+  {
+    cportkey = "-p";
+    sprintf(port_str, "%d", port);
+    cport = port_str;
+  }
+  if ( !is_empty_string(pref.v.smbclient_arg) )
+  {
+    string_ncopy(add_arg_string, pref.v.smbclient_arg, ARG_LEN);
+    add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+  }
   if((pipe_nok=pipe(browsedat->fd))|| (browsedat->childpid =fork())== -1)   
     {
       if (!pipe_nok)
@@ -646,22 +750,55 @@ char group_type[MAXGROUPNAMEL+1+3];
     {
      close(browsedat->fd[0]);   
      dup2(browsedat->fd[1],STDOUT_FILENO); 
-     execlp(pref_get_smbclient_exe(),
+     
+     cArg[argcount++] = "smbclient";
+     cArg[argcount++] = "-L";
+     cArg[argcount++] = master_browser;
+     cArg[argcount++] = "-W";
+     cArg[argcount++] = group_name;
+     cArg[argcount++] = browsedat->userpasswdstr;
+     cArg[argcount++] = "-d1";
+     if ( (cportkey != NULL) && (cport && NULL) ) {
+      cArg[argcount++] = cportkey;
+      cArg[argcount++] = cport;
+     }
+     for ( loop = 0; loop < add_argcount; loop++ )
+     {
+       cArg[argcount++] = cAddArg[loop];
+     }
+     cArg[argcount] = NULL;
+     
+     execvp(pref_get_smbclient_exe(), cArg);
+     
+     /*execlp(pref_get_smbclient_exe(),
             "smbclient",
             "-L",master_browser,
             "-W",group_name,browsedat->userpasswdstr,
             "-d1",
-            NULL);
+            cportkey,
+            cport,
+            NULL);*/
      _exit(0);
     }
     else
     {
      sprintf(logstr,"GetSMBGroups:%s -L %s -W %s %s -d1",pref_get_smbclient_exe(),master_browser,group_name,browsedat->userdummystr);
+     if ( (cport != NULL) && (cportkey != NULL) )
+     {
+       string_ncat(logstr, " ", MAXLOGSTRLEN);
+       string_ncat(logstr, cportkey, MAXLOGSTRLEN);
+       string_ncat(logstr, " ", MAXLOGSTRLEN);
+       string_ncat(logstr, cport, MAXLOGSTRLEN);
+     }
+     for ( loop = 0; loop < add_argcount; loop++ )
+     {
+       string_ncat(logstr, " ", MAXLOGSTRLEN);
+       string_ncat(logstr, cAddArg[loop], MAXLOGSTRLEN);
+     }
      gui_log_window_insert_string(logstr,1);
      close(browsedat->fd[1]);   
      browsedat->firstbrowse=first_master;/*TRUE*/
      browsedat->foundsecondentries=FALSE;
-/*     gbrowsedat.group_master.group_name[0]=0; */
      browsedat->group_master.group_name[MAXGROUPNAMEL]=0;
      strncpy(browsedat->group_master.group_name,group_name,MAXGROUPNAMEL);
      browsedat->group_master.master_browser[MAXMACHNAMEL]=0;
@@ -679,11 +816,42 @@ char group_type[MAXGROUPNAMEL+1+3];
       strcat(group_type,"#1b"); /*#1b= Domain Masterbrowser*/
                                 /*#1d= Local Masterbrowser*/
     }  
+       
+    if ( !is_empty_string(pref.v.nmblookup_arg) )
+    {
+      string_ncopy(add_arg_string, pref.v.nmblookup_arg, ARG_LEN);
+      add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+    }
+     
     if (!browsedat->childpid)
     {
      close(browsedat->fd[0]);   
-     dup2(browsedat->fd[1],STDOUT_FILENO); 
-     if (*wins_name)
+     dup2(browsedat->fd[1],STDOUT_FILENO);
+     
+     cArg[argcount++] = "nmblookup";
+     if ( *wins_name )
+     {
+       cArg[argcount++] = "-U";
+       cArg[argcount++] = wins_name;
+       cArg[argcount++] = "-R";
+       cArg[argcount++] = group_type;
+     }
+     else
+     {
+       cArg[argcount++] = "-M";
+       cArg[argcount++] = group_name;
+     }
+     cArg[argcount++] = "-S";
+     cArg[argcount++] = "-d1";
+     for ( loop = 0; loop < add_argcount; loop++ )
+     {
+       cArg[argcount++] = cAddArg[loop];
+     }
+     cArg[argcount++] = NULL;
+           
+     execvp(pref_get_nmblookup_exe(), cArg);
+     
+     /*if (*wins_name)
        execlp(pref_get_nmblookup_exe(),
               "nmblookup",
               "-U",wins_name,
@@ -697,7 +865,7 @@ char group_type[MAXGROUPNAMEL+1+3];
               "-M",group_name,
               "-S",
               "-d1",
-              NULL);
+              NULL);*/
      _exit(0);
     }
     else
@@ -706,6 +874,11 @@ char group_type[MAXGROUPNAMEL+1+3];
        sprintf(logstr,"GetSMBGroups:%s -U %s -R %s -S -d1",pref_get_nmblookup_exe(),wins_name,group_type);
      else
        sprintf(logstr,"GetSMBGroups:%s -M %s -S -d1",pref_get_nmblookup_exe(),group_name);
+     for ( loop = 0; loop < add_argcount; loop++ )
+     {
+       string_ncat(logstr, " ", MAXLOGSTRLEN);
+       string_ncat(logstr, cAddArg[loop], MAXLOGSTRLEN);
+     }
      gui_log_window_insert_string(logstr,1);
      close(browsedat->fd[1]);   
      browsedat->group_master.group_name[MAXGROUPNAMEL]=0;
@@ -723,8 +896,6 @@ static void GetSMBMachines_inputcallback(HBROWSE_STRUCT *browsedat,gint source,
 {
   int  linec;
   char *posi1,*posi2;
-/*  char machine_name[MAXMACHNAMEL+1],machine_comment[MAXCOMMENTL+1];
-  NEIGHBOR_ENTRY machine; */
   MACHINECOMMENT_STRUCT *machine_comment;
   GSList *machinecommentlist;
   GetSMBStateType smbstate;
@@ -784,9 +955,6 @@ static void GetSMBMachines_inputcallback(HBROWSE_STRUCT *browsedat,gint source,
          {
           browsedat->machinecommentlist=g_slist_append(browsedat->machinecommentlist,
                                                                    machine_comment);
-/*          machine.entry_name=machine_name;
-          machine.entry_comment=machine_comment;
-          add_machine(browsedat->group_master->group_name,&machine,browsedat->mode);*/
          }
         }
         else
@@ -819,7 +987,6 @@ static void GetSMBMachines_inputcallback(HBROWSE_STRUCT *browsedat,gint source,
      machinecommentlist=browsedat->machinecommentlist;
      GetSMBMachines_done(browsedat->group_master.group_name,machinecommentlist,browsedat->browse_info,smbstate);
      slist_free_with_data(&(browsedat->machinecommentlist));
-/*     GetSMBHosts_done(browsedat->group_name,smbstate);*/
      hbrowsedatlist=g_slist_remove(hbrowsedatlist,browsedat);
      g_free(browsedat);
     }
@@ -828,15 +995,29 @@ static void GetSMBMachines_inputcallback(HBROWSE_STRUCT *browsedat,gint source,
 
 
 
-void GetSMBMachines (char * group_name, char *master_browser, char * user, char * passwd,char *domain,MACHINES_BROWSE_INFO *browse_info)
+void GetSMBMachines (char * group_name, char *master_browser, char * user, char * passwd,char *domain,int port,MACHINES_BROWSE_INFO *browse_info)
 {
-//GSList *hbdlist;/*,*machinecommentlist; */
-HBROWSE_STRUCT *browsedat;
-int pipe_nok;
-char logstr[MAXLOGSTRLEN+1];
-char commandstr[MAXCOMMANDL+1];
+  HBROWSE_STRUCT *browsedat;
+  int pipe_nok;
+  char logstr[MAXLOGSTRLEN+1];
+  char commandstr[MAXCOMMANDL+1];
+  char port_str[16];
+  char *cport = NULL;
+  char *cportkey = NULL;
+  char *cArg[20];
+  char *cAddArg[10];
+  int argcount = 0;
+  char add_arg_string[ARG_LEN+1];
+  int add_argcount = 0;
+  int loop;
 
-/*  g_print("GetSMBHosts(%s,%s)\n",group_name,master_browser); */
+  if ( (port >= 0) && (port < 65536) )
+  {
+    cportkey = "-p";
+    sprintf(port_str, "%d", port);
+    cport = port_str;
+  }
+  
   if (!strcmp(master_browser,nomaster))
   {
      GetSMBMachines_done(group_name,NULL,browse_info,SMB_STATE_SMBERR);
@@ -845,35 +1026,18 @@ char commandstr[MAXCOMMANDL+1];
      gui_log_window_insert_string(logstr,1);
      return;
   }
-/*  if (hbrowsedatlist)
-  {
-    hbdlist= hbrowsedatlist;
-    while(hbdlist)
-     {
-      if (strcasecmp(((HBROWSE_STRUCT*)(hbdlist->data))->group_master.group_name,group_name))
-      {
-       hbdlist=hbdlist->next;
-      }
-      else  
-      { //if GetSMBHosts with same group is running, stop it 
-       browsedat=hbdlist->data;
-       close(browsedat->fd[0]);
-       usleep(50000);
-       kill(browsedat->childpid,SIGINT);
-       waitpid(browsedat->childpid,NULL,0);
-       gdk_input_remove(browsedat->input_tag); 
-       hbrowsedatlist=g_slist_remove(hbrowsedatlist,browsedat);
-       slist_free_with_data(&(browsedat->machinecommentlist));
-       g_free(browsedat);
-       break;
-      }
-     }
-  }   */
   browsedat=g_malloc(sizeof(HBROWSE_STRUCT));
   if (*user==0)
     strcpy(commandstr,"-U%"); 
   else
     sprintf(commandstr,"-U%s%%%s",user,passwd);
+   
+  if ( !is_empty_string(pref.v.smbclient_arg) )
+  {
+    string_ncopy(add_arg_string, pref.v.smbclient_arg, ARG_LEN);
+    add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+  }
+   
   if((pipe_nok=pipe(browsedat->fd))|| (browsedat->childpid =fork())== -1)   
     {
       if (!pipe_nok)
@@ -886,21 +1050,28 @@ char commandstr[MAXCOMMANDL+1];
   {
    close(browsedat->fd[0]);   
    dup2(browsedat->fd[1],STDOUT_FILENO); 
-   if (*domain)
-     execlp(pref_get_smbclient_exe(),
-          "smbclient",
-          "-L",master_browser,
-          commandstr,
-          "-W",domain,
-          "-d1",
-          NULL);
-   else
-     execlp(pref_get_smbclient_exe(),
-          "smbclient",
-          "-L",master_browser,
-          commandstr,
-          "-d1",
-          NULL);
+   
+   cArg[argcount++] = "smbclient";
+   cArg[argcount++] = "-L";
+   cArg[argcount++] = master_browser;
+   cArg[argcount++] = commandstr;
+   if (*domain) {
+    cArg[argcount++] = "-W";
+    cArg[argcount++] = domain;
+   }
+   cArg[argcount++] = "-d1";
+   if ( (cportkey != NULL) && (cport && NULL) ) {
+    cArg[argcount++] = cportkey;
+    cArg[argcount++] = cport;
+   }
+   for ( loop = 0; loop < add_argcount; loop++ )
+   {
+     cArg[argcount++] = cAddArg[loop];
+   }
+   
+   cArg[argcount] = NULL;
+     
+   execvp(pref_get_smbclient_exe(), cArg);
    _exit(0);
   }
   else
@@ -910,6 +1081,18 @@ char commandstr[MAXCOMMANDL+1];
      sprintf(logstr,"GetSMBMachines:%s -L %s %s -W %s -d1",pref_get_smbclient_exe(),master_browser,commandstr,domain);
    else
      sprintf(logstr,"GetSMBMachines:%s -L %s %s -d1",pref_get_smbclient_exe(),master_browser,commandstr);
+   if ( (cport != NULL) && (cportkey != NULL) )
+   {
+     string_ncat(logstr, " ", MAXLOGSTRLEN);
+     string_ncat(logstr, cportkey, MAXLOGSTRLEN);
+     string_ncat(logstr, " ", MAXLOGSTRLEN);
+     string_ncat(logstr, cport, MAXLOGSTRLEN);
+   }
+   for ( loop = 0; loop < add_argcount; loop++ )
+   {
+     string_ncat(logstr, " ", MAXLOGSTRLEN);
+     string_ncat(logstr, cAddArg[loop], MAXLOGSTRLEN);
+   }
    gui_log_window_insert_string(logstr,1);
    close(browsedat->fd[1]);   
    hbrowsedatlist=g_slist_append(hbrowsedatlist,browsedat);
@@ -919,6 +1102,7 @@ char commandstr[MAXCOMMANDL+1];
 /*   strcpy(browsedat->group_name,group_name); */
    strcpy(browsedat->group_master.master_browser,master_browser);
    strcpy(browsedat->group_master.group_name,group_name);
+   browsedat->port = port;
    browsedat->browse_info=browse_info;
    browsedat->machinecommentlist=NULL;
    browsedat->lasttab=0;
@@ -1070,40 +1254,25 @@ static void GetSMBShares_inputcallback(SBROWSE_STRUCT *browsedat,gint source, Gd
 }
 
 
-void GetSMBShares (char *group_name,char *machine_name,char *domain,char *ipaddr,char *user,char *passwd,SHARES_BROWSE_INFO *browse_info )
+void GetSMBShares (char *group_name,char *machine_name,char *domain,char *ipaddr,char *user,char *passwd,int port,SHARES_BROWSE_INFO *browse_info )
 {
-char commandstr[MAXCOMMANDL+1];
-//GSList *sbdlist;/*,*sharecommenttypelist; */
-SBROWSE_STRUCT *browsedat;
-int pipe_nok;
-int argnr=-1;
-char *argv[10];
+  char commandstr[MAXCOMMANDL+1];
+  SBROWSE_STRUCT *browsedat;
+  int pipe_nok;
+  int argnr=-1;
+  char *argv[22];
+  char port_str[16];
+  char *cAddArg[10];
+  char add_arg_string[ARG_LEN+1];
+  int add_argcount = 0;
+  int loop;
 
-/*  g_print("GetSMBShare(%s,%s,%s,%s)\n",group_name,machine_name,user,passwd); */
-/*  if (sbrowsedatlist)
+  if ( !is_empty_string(pref.v.smbclient_arg) )
   {
-    sbdlist= sbrowsedatlist;
-    while(sbdlist)
-    {
-     if(strcasecmp(((SBROWSE_STRUCT*)(sbdlist->data))->machine_name,machine_name))
-      {     
-       sbdlist=sbdlist->next;
-      } 
-     else
-     {//if GetSMBShare with same machine is running, stop it 
-      browsedat=sbdlist->data;
-      close(browsedat->fd[0]);
-      usleep(50000);
-      kill(browsedat->childpid,SIGINT);
-     waitpid(browsedat->childpid,NULL,0);
-      gdk_input_remove(browsedat->input_tag); 
-      sbrowsedatlist=g_slist_remove(sbrowsedatlist,browsedat);
-      slist_free_with_data(&(browsedat->sharecommenttypelist));
-      g_free(browsedat);
-      break;
-     }  
-    }
-  }*/
+    string_ncopy(add_arg_string, pref.v.smbclient_arg, ARG_LEN);
+    add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+  }
+  
   if (*user==0)
     strcpy(commandstr,"-U%"); 
   else
@@ -1123,6 +1292,16 @@ char *argv[10];
     argv[++argnr]=domain;
   }
   argv[++argnr]="-d3";/*We want "Connecting to" message*/
+  if ( (port >= 0) && (port < 65536) )
+  {
+    argv[++argnr] = "-p";
+    sprintf(port_str, "%d", port);
+    argv[++argnr] = port_str;
+  }
+  for ( loop = 0; loop < add_argcount; loop++ )
+  {
+    argv[++argnr] = cAddArg[loop];
+  }
   argv[++argnr]=NULL;
   browsedat=g_malloc(sizeof(SBROWSE_STRUCT));
   if((pipe_nok=pipe(browsedat->fd))|| (browsedat->childpid =fork())== -1)   
@@ -1154,6 +1333,7 @@ char *argv[10];
   strcpy(browsedat->group_name,group_name);
   strncpy(browsedat->user, user, USER_LEN);
   strncpy(browsedat->password, passwd, PASSWORD_LEN);
+  browsedat->port = port;
   browsedat->browse_info=browse_info;
   browsedat->sharecommenttypelist=NULL;
   browsedat->machineinfo.ipaddr[0]=0;
@@ -1245,6 +1425,12 @@ GSList *lookuplist;
 int pipe_nok;
 int fd[2];
 char logstr[MAXLOGSTRLEN+1];
+char *cArg[20];
+char *cAddArg[10];
+int argcount = 0;
+char add_arg_string[ARG_LEN+1];
+int add_argcount = 0;
+int loop;
 /*  g_print("LookupByName(%s,%s)\n",machine_name, wins_name); */
 
   if (lookupbynamelist)
@@ -1271,6 +1457,13 @@ char logstr[MAXLOGSTRLEN+1];
     }
   }
   lookupdat=g_malloc(sizeof(LOOKUP_STRUCT));
+  
+  if ( !is_empty_string(pref.v.nmblookup_arg) )
+  {
+    string_ncopy(add_arg_string, pref.v.nmblookup_arg, ARG_LEN);
+    add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+  }
+  
   if((pipe_nok=pipe(fd))|| (lookupdat->childpid =fork())== -1)   
     {
       if (!pipe_nok)
@@ -1283,7 +1476,30 @@ char logstr[MAXLOGSTRLEN+1];
   {
    close(fd[0]);   
    dup2(fd[1],STDOUT_FILENO); 
-   if (*wins_name)
+   
+   cArg[argcount++] = "nmblookup";
+   if ( *wins_name )
+   {
+     cArg[argcount++] = "-U";
+     cArg[argcount++] = wins_name;
+     cArg[argcount++] = "-R";
+     cArg[argcount++] = machine_name;
+   }
+   else
+   {
+     cArg[argcount++] = machine_name;
+   }
+   cArg[argcount++] = "-S";
+   cArg[argcount++] = "-d1";
+   for ( loop = 0; loop < add_argcount; loop++ )
+   {
+     cArg[argcount++] = cAddArg[loop];
+   }
+   cArg[argcount++] = NULL;
+           
+   execvp(pref_get_nmblookup_exe(), cArg);
+   
+   /*if (*wins_name)
      execlp(pref_get_nmblookup_exe(),
            "nmblookup",
             "-U",wins_name,
@@ -1297,7 +1513,7 @@ char logstr[MAXLOGSTRLEN+1];
             machine_name,
             "-S",
             "-d1",
-            NULL);
+            NULL);*/
    _exit(0);
   }
   else
@@ -1306,6 +1522,11 @@ char logstr[MAXLOGSTRLEN+1];
     sprintf(logstr,"LookupByName:%s -U %s -R %s -S -d1",pref_get_nmblookup_exe(),wins_name,machine_name);
   else
     sprintf(logstr,"LookupByName:%s %s -S -d1",pref_get_nmblookup_exe(),machine_name);
+  for ( loop = 0; loop < add_argcount; loop++ )
+  {
+    string_ncat(logstr, " ", MAXLOGSTRLEN);
+    string_ncat(logstr, cAddArg[loop], MAXLOGSTRLEN);
+  }
   gui_log_window_insert_string(logstr,1);
   close(fd[1]);   
   lookupbynamelist=g_slist_append(lookupbynamelist,lookupdat);
@@ -1395,7 +1616,14 @@ GSList *lookuplist;
 int pipe_nok;
 int fd[2];
 char logstr[MAXLOGSTRLEN+1];
+char *cArg[20];
+char *cAddArg[10];
+int argcount = 0;
+char add_arg_string[ARG_LEN+1];
+int add_argcount = 0;
+int loop;
 /*  g_print("LookupByIP(%s)\n",ipaddr); */
+  
   if (lookupbyiplist)
   {
     lookuplist= lookupbyiplist;
@@ -1420,6 +1648,13 @@ char logstr[MAXLOGSTRLEN+1];
     }
   }
   lookupdat=g_malloc(sizeof(LOOKUP_STRUCT));
+  
+  if ( !is_empty_string(pref.v.nmblookup_arg) )
+  {
+    string_ncopy(add_arg_string, pref.v.nmblookup_arg, ARG_LEN);
+    add_argcount = tokenize_string(cAddArg, add_arg_string, 10);
+  }
+  
   if((pipe_nok=pipe(fd))|| (lookupdat->childpid =fork())== -1)   
     {
       if (!pipe_nok)
@@ -1432,16 +1667,35 @@ char logstr[MAXLOGSTRLEN+1];
   {
    close(fd[0]);   
    dup2(fd[1],STDOUT_FILENO); 
-   execlp(pref_get_nmblookup_exe(),
+   
+   cArg[argcount++] = "nmblookup";
+   cArg[argcount++] = "-A";
+   cArg[argcount++] = ipaddr;
+   cArg[argcount++] = "-d1";
+   
+   for ( loop = 0; loop < add_argcount; loop++ )
+   {
+     cArg[argcount++] = cAddArg[loop];
+   }
+   cArg[argcount++] = NULL;
+           
+   execvp(pref_get_nmblookup_exe(), cArg);
+   
+   /*execlp(pref_get_nmblookup_exe(),
          "nmblookup",
           "-A",ipaddr,
           "-d1",
-          NULL);
+          NULL);*/
    _exit(0);
   }
   else
   {
   sprintf(logstr,"LookupByIP:%s -A %s -d1",pref_get_nmblookup_exe(),ipaddr);
+  for ( loop = 0; loop < add_argcount; loop++ )
+  {
+    string_ncat(logstr, " ", MAXLOGSTRLEN);
+    string_ncat(logstr, cAddArg[loop], MAXLOGSTRLEN);
+  }
   gui_log_window_insert_string(logstr,1);
   close(fd[1]);   
   lookupbyiplist=g_slist_append(lookupbyiplist,lookupdat);
