@@ -54,6 +54,7 @@ typedef struct _MOUNT_STRUCT{
   char stdoutbuf[MAXERRLEN+1];
   int  linepos,linepos2,linenum,oneeof,cleanup;
   int  sumount,passw_lf;
+  int  port;
 } MOUNT_STRUCT;
 static GSList *mountlist = NULL;
 
@@ -70,13 +71,9 @@ typedef struct _UMOUNT_STRUCT{
 static GSList *umountlist = NULL;
 
 
-/*char smbmountExe[PATH_MAX]  = "smbmount";
-char smbumountExe[PATH_MAX] = "smbumount";*/
-
 char suprompt[] = "Passw";
 
 unsigned char linux2_0=FALSE;  /*Get Version-Info at runtime, 1=kernel<2.1.70*/
-/*SMBMOUNT smbmount_version=SMBMOUNT_204;*/  /*default smbmount from samba < 2.0.5*/
 
 char *escape_str(char *s,GSList **strlist) /*Add escape characters to strings */
 {                                          /*  and collect allocated space in strlist*/
@@ -316,7 +313,7 @@ static void smbmount_inputcallback(MOUNT_STRUCT *mountdat,gint source, GdkInputC
 
 void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr,
               char *group_name,char *smbuser,char *smbpasswd,uid_t uid,gid_t gid,
-              mode_t filemode,mode_t dirmode,char *supasswd)
+              mode_t filemode,mode_t dirmode,int port,char *supasswd)
 {
  char notfound[]="smbmount not found\n";
  int fdstdin[2],i;
@@ -338,6 +335,9 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
  MOUNT_STRUCT *mountdat;
  char *argv[MAX_ARG+1];
  GSList *strlist=NULL;
+ char port_str[16];
+ 
+ sprintf(port_str, "%d", port);
  
  memset(dummysmbpasswd,'*',sizeof(dummysmbpasswd)-1);
  if (strlen(smbpasswd)<sizeof(dummysmbpasswd)) dummysmbpasswd[strlen(smbpasswd)]=0;
@@ -370,17 +370,19 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
     else *opt_group=0;
     if (linux2_0)
     {
-    if (*smbpasswd==0)
-     sprintf(commandstr,"%s %s %s -c %s -I %s %s -U %s -f %d -d %d -u %d -g %d -n",
-       escape_str(pref_get_smbmount_exe(),&strlist),escape_str(servicename,&strlist),escape_str(mount_point,&strlist),
-       escape_str(clientname,&strlist),ipaddr,opt_group,escape_str(smbuser,&strlist),filemode,dirmode,uid,gid);
-    else
-     sprintf(commandstr,"%s %s %s -c %s -I %s %s -U %s -f %d -d %d -u %d -g %d -P %s",
-      escape_str(pref_get_smbmount_exe(),&strlist),escape_str(servicename,&strlist),escape_str(mount_point,&strlist),escape_str(clientname,&strlist),
-      ipaddr,opt_group,escape_str(smbuser,&strlist),filemode,dirmode,uid,gid,escape_str(smbpasswd,&strlist));
+      /* linux 2.0 */
+      if (*smbpasswd==0)
+       sprintf(commandstr,"%s %s %s -c %s -I %s %s -U %s -f %d -d %d -u %d -g %d -n",
+         escape_str(pref_get_smbmount_exe(),&strlist),escape_str(servicename,&strlist),escape_str(mount_point,&strlist),
+         escape_str(clientname,&strlist),ipaddr,opt_group,escape_str(smbuser,&strlist),filemode,dirmode,uid,gid);
+      else
+       sprintf(commandstr,"%s %s %s -c %s -I %s %s -U %s -f %d -d %d -u %d -g %d -P %s",
+        escape_str(pref_get_smbmount_exe(),&strlist),escape_str(servicename,&strlist),escape_str(mount_point,&strlist),escape_str(clientname,&strlist),
+        ipaddr,opt_group,escape_str(smbuser,&strlist),filemode,dirmode,uid,gid,escape_str(smbpasswd,&strlist));
     }
     else
     { 
+       /* linux 2.2/2.4 */
        switch (pref.v.smbmount_version ){
        case SMBMOUNT_205:
         sprintf(commandstr,"%s %s %s -I %s %s -U%s%%%s -d0",
@@ -392,6 +394,16 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
          escape_str(pref_get_smbmount_exe(),&strlist),escape_str(servicename,&strlist),escape_str(mount_point,&strlist),
          escape_str(smbuser,&strlist),escape_str(smbpasswd,&strlist),
          filemode,dirmode,uid,gid,ipaddr,opt_group);
+         if ( (port >= 0) && (port < 65536) )
+         {
+          sprintf(commandstr, "%s,port=%d", commandstr, port);
+         }
+         if ( !is_empty_string(pref.v.smbmount_arg) )
+         {
+           if ( pref.v.smbmount_arg[0] != ',' )
+            string_ncat(commandstr, ",", MAXMNTCOMMANDL);
+           string_ncat(commandstr, escape_str(pref.v.smbmount_arg,&strlist), MAXMNTCOMMANDL);
+         }
         break;
        default: /*SMBMOUNT_204*/
         sprintf(commandstr1,"mount %s -f %d -d %d -u %d -g %d",/*escape_str_smbmnt(*/mount_point/*,&strlist)*/,filemode,dirmode,uid,gid);
@@ -405,85 +417,101 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
   }
   else
   {
+    /* no sumount */
     if(linux2_0)
     {
-    argv[++argnr]="smbmount";
-    argv[++argnr]=servicename;
-    argv[++argnr]=mount_point;
-    argv[++argnr]="-c";
-    argv[++argnr]=clientname;
-    argv[++argnr]="-I";
-    argv[++argnr]=ipaddr;
-    if (*group_name)
-    {
-      argv[++argnr]="-D";
-      argv[++argnr]=group_name;
-    }
-    argv[++argnr]="-U";
-    argv[++argnr]=smbuser;
-    argv[++argnr]="-f";sprintf(filemodestr,"%d",filemode);
-    argv[++argnr]=filemodestr;
-    argv[++argnr]="-d";sprintf(dirmodestr,"%d",dirmode);
-    argv[++argnr]=dirmodestr;
-    argv[++argnr]="-u";sprintf(uidstr,"%d",uid);
-    argv[++argnr]=uidstr;
-    argv[++argnr]="-g";sprintf(gidstr,"%d",gid);
-    argv[++argnr]=gidstr;
-    if (*smbpasswd==0)
-      argv[++argnr]="-n";
-    else
-    {
-      argv[++argnr]="-P";
-      argv[++argnr]=smbpasswd;
-    }
-    argv[++argnr]=NULL;
-    }
-    else
-    {
-    argv[++argnr]=pref_get_smbmount_exe();
-    argv[++argnr]=servicename;
-    if (pref.v.smbmount_version>SMBMOUNT_204) argv[++argnr]=mount_point;
-    if (pref.v.smbmount_version<SMBMOUNT_206){
+      argv[++argnr]="smbmount";
+      argv[++argnr]=servicename;
+      argv[++argnr]=mount_point;
+      argv[++argnr]="-c";
+      argv[++argnr]=clientname;
       argv[++argnr]="-I";
       argv[++argnr]=ipaddr;
       if (*group_name)
       {
-        argv[++argnr]="-W";
+        argv[++argnr]="-D";
         argv[++argnr]=group_name;
       }
-      sprintf(userpasswd,"-U%s%%%s",smbuser,smbpasswd);
-      argv[++argnr]=userpasswd;
-      argv[++argnr]="-d0";
-      if (pref.v.smbmount_version <SMBMOUNT_205)
+      argv[++argnr]="-U";
+      argv[++argnr]=smbuser;
+      argv[++argnr]="-f";sprintf(filemodestr,"%d",filemode);
+      argv[++argnr]=filemodestr;
+      argv[++argnr]="-d";sprintf(dirmodestr,"%d",dirmode);
+      argv[++argnr]=dirmodestr;
+      argv[++argnr]="-u";sprintf(uidstr,"%d",uid);
+      argv[++argnr]=uidstr;
+      argv[++argnr]="-g";sprintf(gidstr,"%d",gid);
+      argv[++argnr]=gidstr;
+      if (*smbpasswd==0)
+        argv[++argnr]="-n";
+      else
       {
-       argv[++argnr]="-c";
-       argv[++argnr]=commandstr;
-       sprintf(commandstr,"mount %s -f %d -d %d -u %d -g %d",
-                /*escape_str_smbmnt(*/mount_point/*,&strlist)*/,filemode,dirmode,uid,gid);
+        argv[++argnr]="-P";
+        argv[++argnr]=smbpasswd;
       }
-     }
-     else /*>=SMBMOUNT_206*/
-     {
-      argv[++argnr]="-o";
-      argv[++argnr]=commandstr;
-/*      sprintf(commandstr,"username=%s%%%s,fmask=%d,dmask=%d,uid=%d,gid=%d,ip=%s,debug=0",
-         escape_str(smbuser,&strlist),escape_str(smbpasswd,&strlist),
-         filemode,dirmode,uid,gid,ipaddr);*/
-      sprintf(commandstr,"username=%s%%%s,fmask=%d,dmask=%d,uid=%d,gid=%d,ip=%s,debug=0",
-         smbuser,smbpasswd,filemode,dirmode,uid,gid,ipaddr);
-      if (*group_name){
-        strcat(commandstr,",workgroup=");
-        strcat(commandstr,escape_str(group_name,&strlist));
-      }
-     }
-     argv[++argnr]=NULL;
+      argv[++argnr]=NULL;
     }
+    else
+    {
+      /* linux 2.2/2.4 */
+      argv[++argnr]=pref_get_smbmount_exe();
+      argv[++argnr]=servicename;
+      if (pref.v.smbmount_version>SMBMOUNT_204) argv[++argnr]=mount_point;
+      if (pref.v.smbmount_version<SMBMOUNT_206)
+      {
+        /* 2.04/2.05 */
+        argv[++argnr]="-I";
+        argv[++argnr]=ipaddr;
+        if (*group_name)
+        {
+          argv[++argnr]="-W";
+          argv[++argnr]=group_name;
+        }
+        sprintf(userpasswd,"-U%s%%%s",smbuser,smbpasswd);
+        argv[++argnr]=userpasswd;
+        argv[++argnr]="-d0";
+        if (pref.v.smbmount_version <SMBMOUNT_205)
+        {
+         argv[++argnr]="-c";
+         argv[++argnr]=commandstr;
+         sprintf(commandstr,"mount %s -f %d -d %d -u %d -g %d",
+                /*escape_str_smbmnt(*/mount_point/*,&strlist)*/,filemode,dirmode,uid,gid);
+        }
+        if ( !is_empty_string(pref.v.smbmount_arg) )
+        {
+        }
+      }
+      else /*>=SMBMOUNT_206*/
+      {
+        argv[++argnr]="-o";
+        argv[++argnr]=commandstr;
+        sprintf(commandstr,"username=%s%%%s,fmask=%d,dmask=%d,uid=%d,gid=%d,ip=%s,debug=0",
+           smbuser,smbpasswd,filemode,dirmode,uid,gid,ipaddr);
+        if (*group_name)
+        {
+          strcat(commandstr,",workgroup=");
+          strcat(commandstr,escape_str(group_name,&strlist));
+        }
+        if ( (port >= 0) && (port < 65536) )
+        {
+          string_ncat(commandstr, ",port=", MAXMNTCOMMANDL);
+          string_ncat(commandstr, port_str, MAXMNTCOMMANDL);
+        }
+        if ( !is_empty_string(pref.v.smbmount_arg) )
+        {
+          if ( pref.v.smbmount_arg[0] != ',' )
+           string_ncat(commandstr, ",", MAXMNTCOMMANDL);
+          string_ncat(commandstr, escape_str(pref.v.smbmount_arg,&strlist), MAXMNTCOMMANDL);
+        }
+       }
+       argv[++argnr]=NULL;
+      }
   }
   slist_free_with_data(&strlist);
   mountdat=g_malloc(sizeof(MOUNT_STRUCT));
   if((pipe2_nok=pipe(fdstdout))|| (pipe3_nok=pipe(fdstderr))||
         (mountdat->childpid =fork())== -1)   
-   {
+  {
     if (sumount|| passw_lf){close(fdstdin[0]);close(fdstdin[1]);}
     if (!pipe2_nok) 
       {close(fdstdout[0]);close(fdstdout[1]);}
@@ -492,7 +520,8 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
     g_free(mountdat);
     g_print(_("smbmount pipe,fork error\n"));
     return;
-   } 
+  } 
+  
   if (!mountdat->childpid)
   {
    close(fdstdout[0]);   
@@ -547,6 +576,17 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
         sprintf(commandstr,"%s %s %s -o username=%s%%%s,fmask=%d,dmask=%d,uid=%d,gid=%d,ip=%s,debug=0%s",
          pref_get_smbmount_exe(),servicename,mount_point,smbuser,dummysmbpasswd,
          filemode,dirmode,uid,gid,ipaddr,opt_group);
+        if ( (port >= 0) && (port < 65536) )
+        {
+          string_ncat(commandstr, ",port=", MAXMNTCOMMANDL);
+          string_ncat(commandstr, port_str, MAXMNTCOMMANDL);
+        }
+        if ( !is_empty_string(pref.v.smbmount_arg) )
+        {
+          if ( pref.v.smbmount_arg[0] != ',' )
+           string_ncat(commandstr, ",", MAXMNTCOMMANDL);
+          string_ncat(commandstr, escape_str(pref.v.smbmount_arg,&strlist), MAXMNTCOMMANDL);
+        }
         break;
       default:
         sprintf(commandstr,"'%s' '%s' -I %s %s '-U%s%%%s' -d0 -c 'mount %s -f %d -d %d -u %d -g %d'",
@@ -587,6 +627,17 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
          strcat(commandstr,",workgroup=");
          strcat(commandstr,group_name);
        }
+       if ( (port >= 0) && (port < 65536) )
+       {
+         string_ncat(commandstr, ",port=", MAXMNTCOMMANDL);
+         string_ncat(commandstr, port_str, MAXMNTCOMMANDL);
+       }
+       if ( !is_empty_string(pref.v.smbmount_arg) )
+       {
+         if ( pref.v.smbmount_arg[0] != ',' )
+          string_ncat(commandstr, ",", MAXMNTCOMMANDL);
+         string_ncat(commandstr, escape_str(pref.v.smbmount_arg,&strlist), MAXMNTCOMMANDL);
+       }
       }
       if (passw_lf)
       {
@@ -612,6 +663,7 @@ void smbmount(char *servicename, char *mount_point,char *clientname,char *ipaddr
   mountdat->oneeof=FALSE;
   mountdat->cleanup=FALSE;
   mountdat->servicename[PATH_MAX]=0;
+  mountdat->port = port;
   strncpy(mountdat->servicename,servicename,PATH_MAX);
   mountdat->mount_point[PATH_MAX]=0;
   strncpy(mountdat->mount_point,mount_point,PATH_MAX);
@@ -840,13 +892,26 @@ void smbumount(char *mount_point,char *supasswd,char smb,int tag)
     argv[2]=commandstr;
     argv[3]=NULL;
     sprintf(commandstr,"%s %s", escape_str(smb ? pref_get_smbumount_exe():"umount",&strlist),escape_str(mount_point,&strlist));
+    if ( !is_empty_string(pref.v.smbumount_arg) )
+    {
+      string_ncat(commandstr, " ", MAXMNTCOMMANDL);
+      string_ncat(commandstr, escape_str(pref.v.smbumount_arg,&strlist), MAXMNTCOMMANDL);
+    }
     slist_free_with_data(&strlist);
   }
   else
   {
     argv[0]=smb ? "smbumount":"umount";
     argv[1]=mount_point;
-    argv[2]=NULL;
+    if ( !is_empty_string(pref.v.smbumount_arg) )
+    {
+      argv[2] = pref.v.smbumount_arg;
+      argv[3] = NULL;
+    }
+    else
+    {
+      argv[2]=NULL;
+    }
   }
   if((pipe2_nok=pipe(fdstdout))|| (pipe3_nok=pipe(fdstderr))||
         (childpid =fork())== -1)   
