@@ -229,6 +229,80 @@ void string_replace_char_by_char (char *str, char old, char replace)
   }
 }
 
+/*
+ *  tokenizes a string by whitespaces.
+ *  take a look to " and ' characters.
+ */
+
+int tokenize_string(char **dest, char *src, int max_token)
+{
+  char *next;
+  int length;
+  int token_count = 0;
+  unsigned char do_further;
+  
+  *dest = NULL;
+  if ( !src )
+    return 0;
+  
+  while ( token_count < (max_token - 1) )
+  {
+    string_trim(&src);
+    length = strlen(src);
+    if ( !length )
+      break;
+    
+    do_further = 0;
+    
+    if ( *src == '"' )
+    {
+      src++;
+      next = read_until_char(src, '"');
+      if ( *next == '"' )
+      {
+        *next = 0;
+        do_further = 1;
+      }
+      
+    }
+    else if ( *src == '\'' )
+    {
+      src++;
+      next = read_until_char(src, '\'');
+      if ( *next == '\'' )
+      {
+        *next = 0;
+        do_further = 1;
+      }
+    }
+    else
+    {
+      next = read_until_whitespace(src);
+      if ( *next != 0 )
+        do_further = 1;
+    }
+    
+    dest[token_count++] = src;
+    
+    if ( !do_further )
+    {
+      break;
+    }
+    
+/*    if ( *next == 0 )
+    {
+      break;
+    }*/
+    
+    *next = 0;
+    src = next + 1;
+  }
+  
+  dest[token_count] = NULL;
+  
+  return token_count;
+}
+
 static unsigned char need_escaping[] = { ' ', '\t', '\n', '\\' };
 
 char *mangle(unsigned char *s)
@@ -714,12 +788,13 @@ void utility_do_mount (mem_mount_struct *mount)
   }
   argv[argc] = NULL;
   
+  /* makes an entry in the memorized mounts list */
+  data_mem_mount_add(mem_mount_user, mount->mountpoint, &argv[1]);
+  
   /* share already mounted ? */
   if ( mount_list_search_by_mountpoint(mount->mountpoint) != NULL )
   {
     already_mounted = 1;
-    /* makes an entry in the memorized mounts list */
-    data_mem_mount_add(mem_mount_user, mount->mountpoint, &argv[1]);
   }
 
   /* do mount action */
@@ -942,7 +1017,7 @@ void postmount_execute (char *mountpoint)
     }
     /* grandchild process */
     execvp(argv[0], argv);
-    g_error(_("error running programm !\n"));
+    g_error(_("error running program !\n"));
     _exit(1);
   }
   /* parent is waiting for child */
